graphdb-workbench

Bundling

The webpack build pipeline that bundles and minifies the workbench’s scripts, styles, images, and fonts.

Bundling

The workbench application consists of many resources: script files, stylesheets, images, fonts. In a non optimized build this would lead to hundreds of http requests. Nowadays web applications are usually optimized by bundling their resources and minifying them. The reason for this is to have significantly fewer http requests which leads to faster initial load and less traffic.

Bundling of the workbench is done with webpack. It’s configured with three config files:

The common config contains shared configuration for the other two.

There are two ways the application to be built. First is for production and the other is for dev. See the Build and Dev server sections above.

The code for production is build in in the /dist folder. The bundling covers the following tasks:


See also: Developers Guide