Icons
The two icon sets used in GraphDB Workbench: Remix icons and a precompiled Icomoon pack with custom icons.
There are two icon sets used in the project:
- Remix icons https://remixicon.com/
- Icomoon icons is a precompiled version that includes our own custom icons that are missing in the Remix icons
- set. Information on how the iconset can be updated and recompiled can be found here
- https://github.com/Ontotext-AD/ontotext-ui-guidelines/tree/master/ontotext-icon-pack
See also: Developers Guide